from django_celery_beat.models import CrontabSchedule
from rest_framework.response import Response
from rest_framework.views import APIView

from django_celery_task_broker.serializers import CrontabScheduleSerializer


class CrontabScheduleView(APIView):

    def post(self, request):

        serializer = CrontabScheduleSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        data = serializer.data

        crontab_schedule, _ = CrontabSchedule.objects.get_or_create(
            minute=data['minute'],
            hour=data['hour'],
            day_of_week=data['day_of_week'],
            day_of_month=data['day_of_month'],
            month_of_year=data['month_of_year']
        )

        response_data = CrontabScheduleSerializer(crontab_schedule).data

        return Response(response_data)



from django.test import TestCase

from django_celery_beat.models import CrontabSchedule
from model_mommy import mommy

from django_celery_task_broker.serializers import CrontabScheduleSerializer


class ContrabScheduleSerializerTest(TestCase):

    def test_serializer_should_return_all_field(self):
        schedule = mommy.make(CrontabSchedule)
        serializer = CrontabScheduleSerializer(schedule).data

        actual = {key for key in serializer.keys()}

        expected = {
            'id',
            'minute',
            'hour',
            'day_of_week',
            'day_of_month',
            'month_of_year',
        }
        self.assertEqual(actual, expected)



from django.conf.urls import url

from django_celery_task_broker.views.trigger_task_view import TriggerTaskView
from django_celery_task_broker.views.periodic_task_view import PeriodicTaskView
from django_celery_task_broker.views.crontab_schedule_view import CrontabScheduleView


urlpatterns = [
    url(r'^trigger-task/$', TriggerTaskView.as_view(), name='trigger_task'),
    url(r'^periodic-task/$', PeriodicTaskView.as_view(), name='periodic_task'),
    url(r'^crontab-schedule/$', CrontabScheduleView.as_view(), name='crontab_schedule')
]



from celery import current_app


def is_task_exist(module_name: str, task_name: str) -> bool:
    return combine_task_path(module_name, task_name) in current_app.tasks


def combine_task_path(module_name: str, task_name: str) -> str:
    return ('.').join([module_name, task_name])



from django.test import TestCase

from django_celery_task_broker.serializers import TriggerPeriodicTaskSerializer


class TriggerPeriodicTaskSerializerTest(TestCase):

    def setUp(self):
        self.company_account = 'http://gateway:8000/api/account/1000/'
        self.module_name = 'answers.tasks'
        self.task_name = 'active_withheld_answer'
        self.cron_id = '1'
        self.display_name = 'Test Dynamic Create Periodic Task'
        self.kwargs = {'company_account': self.company_account}

        self.serializer = TriggerPeriodicTaskSerializer

    def test_serializer_should_return_defined_fields(self):
        data = {
            'module_name': self.module_name,
            'task_name': self.task_name,
            'display_name': self.display_name,
            'cron_id': self.cron_id,
            'kwargs': self.kwargs
        }
        serializer = self.serializer(data).data

        actual = {key for key in serializer.keys()}

        expected = {
            'display_name',
            'module_name',
            'task_name',
            'cron_id',
            'kwargs'
        }

        self.assertEqual(actual, expected)

    def test_serializer_should_require_all_required_field(self):
        data = {}
        expected_error = {
            'display_name': ['This field is required.'],
            'module_name': ['This field is required.'],
            'task_name': ['This field is required.'],
            'cron_id': ['This field is required.'],
            'kwargs': ['This field is required.']

        }

        actual = self.serializer(data=data)

        self.assertFalse(actual.is_valid())
        self.assertDictEqual(actual.errors, expected_error)



from importlib import import_module

from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from django_celery_task_broker.utils import is_task_exist
from django_celery_task_broker.serializers import TriggerTaskSerializer


class TriggerTaskView(APIView):

    def post(self, request):
        serializer = TriggerTaskSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        data = serializer.data

        if is_task_exist(data['module_name'], data['task_name']):
            module = import_module(data['module_name'])
            getattr(module, data['task_name']).apply_async(kwargs=data['kwargs'])
            return Response(data)

        return Response(status=status.HTTP_400_BAD_REQUEST, data={'detail': 'Invalid Task'})



from django.test import TestCase

from django_celery_task_broker.utils import is_task_exist


class UtilsTaskViewTest(TestCase):

    def test_is_task_exist_with_valid_task_should_return_true(self):
        module_name = 'project.celery'
        task_name = 'example_task'

        self.assertTrue(is_task_exist(module_name, task_name))

    def test_is_task_exist_with_valid_task_should_return_false(self):
        module_name = 'invalid.tasks'
        task_name = 'active_withheld_answer'

        self.assertFalse(is_task_exist(module_name, task_name))



import json

from django.shortcuts import get_object_or_404

from django_celery_beat.models import CrontabSchedule, PeriodicTask
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from django_celery_task_broker.serializers import TriggerPeriodicTaskSerializer, PatchPeriodicTaskSerializer
from django_celery_task_broker.utils import combine_task_path, is_task_exist


class PeriodicTaskView(APIView):

    def post(self, request):
        serializer = TriggerPeriodicTaskSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        data = serializer.data

        schedule = CrontabSchedule.objects.get(id=data['cron_id'])

        if is_task_exist(data['module_name'], data['task_name']):
            task_property = dict(
                crontab=schedule,
                name=data['display_name'],
                task=combine_task_path(data['module_name'], data['task_name']),
                kwargs=json.dumps(data['kwargs'])
            )
            if PeriodicTask.objects.filter(**task_property).exists():
                return Response(
                    status=status.HTTP_200_OK,
                    data={'detail': f"Task:{task_property['task']} already exists"}
                )
            task = PeriodicTask.objects.create(**task_property)
            return Response(status=status.HTTP_200_OK, data={'detail': f'CREATE: {str(task)}'})

        return Response(status=status.HTTP_400_BAD_REQUEST, data={'detail': 'Invalid Task'})

    def patch(self, request):
        serializer = PatchPeriodicTaskSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        data = serializer.data

        task = get_object_or_404(
            PeriodicTask,
            task=combine_task_path(data['module_name'], data['task_name']),
            kwargs__contains=json.dumps(data['kwargs'])
        )
        task.enabled = data['enabled']
        task.save()

        return Response(status=status.HTTP_200_OK, data={'detail': f'Updated: Enabled: {data["enabled"]}: {task}'})



import os
from setuptools import find_packages, setup


os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-celery-task-broker',
    version='0.4.0',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',
    description='A dynamic Django celery task broker',
    url='https://github.com/prontotools/django-celery-task-broker',
    author='Prontotools',
    author_email='prontotools@prontomarketing.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=['django_celery_beat>=1.1.1', 'djangorestframework>=3.6.3']
)



from django_celery_beat.models import CrontabSchedule
from rest_framework import serializers


class TriggerPeriodicTaskSerializer(serializers.Serializer):
    display_name = serializers.CharField(required=True)
    module_name = serializers.CharField(required=True)
    task_name = serializers.CharField(required=True)
    kwargs = serializers.JSONField(required=True)
    cron_id = serializers.CharField(required=True)


class PatchPeriodicTaskSerializer(serializers.Serializer):
    module_name = serializers.CharField(required=True)
    task_name = serializers.CharField(required=True)
    kwargs = serializers.JSONField(required=True)
    enabled = serializers.BooleanField(required=True)


class TriggerTaskSerializer(serializers.Serializer):
    module_name = serializers.CharField(required=True)
    task_name = serializers.CharField(required=True)
    kwargs = serializers.JSONField(required=True)


class CrontabScheduleSerializer(serializers.ModelSerializer):

    class Meta:
        model = CrontabSchedule
        fields = '__all__'
